@@ -43,8 +43,8 @@
 
 #define VERSION_MAJOR 1
 #define VERSION_MINOR 8
-#define VERSION_MICRO 2
-static char pycairo_version_string[] = "1.8.2";
+#define VERSION_MICRO 6
+static char pycairo_version_string[] = "1.8.6";
 
 
 /* A module specific exception */
@@ -88,7 +88,10 @@ Pycairo_Check_Status (cairo_status_t status)
  */
 static Pycairo_CAPI_t CAPI = {
     &PycairoContext_Type,      PycairoContext_FromContext,
-    &PycairoFontFace_Type,     PycairoFontFace_FromFontFace,
+
+    &PycairoFontFace_Type,
+    &PycairoToyFontFace_Type,  PycairoFontFace_FromFontFace,
+
     &PycairoFontOptions_Type,  PycairoFontOptions_FromFontOptions,
     &PycairoMatrix_Type,       PycairoMatrix_FromMatrix,
     &PycairoPath_Type,         PycairoPath_FromPath,
@@ -164,12 +167,16 @@ init_cairo(void)
 {
     PyObject *m;
 
+    /* initialise 'tp_base' here to work round problem with MinGW compiler */
     PycairoContext_Type.tp_base = &PyBaseObject_Type;
     if (PyType_Ready(&PycairoContext_Type) < 0)
         return;
     PycairoFontFace_Type.tp_base = &PyBaseObject_Type;
     if (PyType_Ready(&PycairoFontFace_Type) < 0)
         return;
+    PycairoToyFontFace_Type.tp_base = &PycairoFontFace_Type;
+    if (PyType_Ready(&PycairoToyFontFace_Type) < 0)
+        return;
     PycairoFontOptions_Type.tp_base = &PyBaseObject_Type;
     if (PyType_Ready(&PycairoFontOptions_Type) < 0)
         return;
@@ -249,6 +256,8 @@ init_cairo(void)
     PyModule_AddObject(m, "Context", (PyObject *)&PycairoContext_Type);
     Py_INCREF(&PycairoFontFace_Type);
     PyModule_AddObject(m, "FontFace",(PyObject *)&PycairoFontFace_Type);
+    Py_INCREF(&PycairoToyFontFace_Type);
+    PyModule_AddObject(m, "ToyFontFace",(PyObject *)&PycairoToyFontFace_Type);
     Py_INCREF(&PycairoFontOptions_Type);
     PyModule_AddObject(m, "FontOptions",(PyObject *)&PycairoFontOptions_Type);
     Py_INCREF(&PycairoMatrix_Type);
@@ -433,7 +442,6 @@ init_cairo(void)
     CONSTANT(FORMAT_RGB24);
     CONSTANT(FORMAT_A8);
     CONSTANT(FORMAT_A1);
-    CONSTANT(FORMAT_RGB16_565);
 
     CONSTANT(HINT_METRICS_DEFAULT);
     CONSTANT(HINT_METRICS_OFF);
