@@ -2,7 +2,7 @@
  *
  * Pycairo - Python bindings for cairo
  *
- * Copyright © 2003-2005 James Henstridge
+ * Copyright © 2003 James Henstridge, Steven Chaplin
  *
  * This library is free software; you can redistribute it and/or
  * modify it either under the terms of the GNU Lesser General Public
@@ -37,27 +37,6 @@
 #include "pycairo-private.h"
 
 
-/* Take a PyBaseString (str or unicode) object and return a pointer to the
- * UTF-8 encoded string.
- */
-char *
-__PyBaseString_AsUTF8 (PyObject *o)
-{
-    if (PyString_Check(o)) {
-	/* A plain ASCII string is also a valid UTF-8 string */
-	return PyString_AsString(o);
-
-    } else if (PyUnicode_Check(o)) {
-	PyObject *u = PyUnicode_AsUTF8String(o);
-	if (u != NULL) {
-	    char *utf8 = PyString_AsString(u);
-	    Py_DECREF(u);
-	    return utf8;
-	}
-    }
-    return NULL;
-}
-
 /* PycairoContext_FromContext
  * Create a new PycairoContext from a cairo_t
  * ctx  - a cairo_t to 'wrap' into a Python object.
@@ -124,7 +103,9 @@ pycairo_append_path (PycairoContext *o, PyObject *args)
 			  &PycairoPath_Type, &p))
 	return NULL;
 
+    Py_BEGIN_ALLOW_THREADS
     cairo_append_path (o->ctx, p->path);
+    Py_END_ALLOW_THREADS
     RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
     Py_RETURN_NONE;
 }
@@ -160,7 +141,9 @@ pycairo_arc_negative (PycairoContext *o, PyObject *args)
 static PyObject *
 pycairo_clip (PycairoContext *o)
 {
+    Py_BEGIN_ALLOW_THREADS
     cairo_clip (o->ctx);
+    Py_END_ALLOW_THREADS
     RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
     Py_RETURN_NONE;
 }
@@ -177,7 +160,9 @@ pycairo_clip_extents (PycairoContext *o)
 static PyObject *
 pycairo_clip_preserve (PycairoContext *o)
 {
+    Py_BEGIN_ALLOW_THREADS
     cairo_clip_preserve (o->ctx);
+    Py_END_ALLOW_THREADS
     RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
     Py_RETURN_NONE;
 }
@@ -185,14 +170,13 @@ pycairo_clip_preserve (PycairoContext *o)
 static PyObject *
 pycairo_close_path (PycairoContext *o)
 {
+    Py_BEGIN_ALLOW_THREADS
     cairo_close_path (o->ctx);
+    Py_END_ALLOW_THREADS
     RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
     Py_RETURN_NONE;
 }
 
-/* a possible candidate for an iterator (like cairo_copy_path), but for the
- * typical rectangle_list a simple Python tuple in fine?
- */
 static PyObject *
 pycairo_copy_clip_rectangle_list (PycairoContext *o)
 {
@@ -226,7 +210,9 @@ pycairo_copy_clip_rectangle_list (PycairoContext *o)
 static PyObject *
 pycairo_copy_page (PycairoContext *o)
 {
+    Py_BEGIN_ALLOW_THREADS
     cairo_copy_page (o->ctx);
+    Py_END_ALLOW_THREADS
     RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
     Py_RETURN_NONE;
 }
@@ -234,13 +220,21 @@ pycairo_copy_page (PycairoContext *o)
 static PyObject *
 pycairo_copy_path (PycairoContext *o)
 {
-    return PycairoPath_FromPath (cairo_copy_path (o->ctx));
+    cairo_path_t *cp;
+    Py_BEGIN_ALLOW_THREADS
+    cp = cairo_copy_path (o->ctx);
+    Py_END_ALLOW_THREADS
+    return PycairoPath_FromPath (cp);
 }
 
 static PyObject *
 pycairo_copy_path_flat (PycairoContext *o)
 {
-    return PycairoPath_FromPath (cairo_copy_path_flat (o->ctx));
+    cairo_path_t *cp;
+    Py_BEGIN_ALLOW_THREADS
+    cp = cairo_copy_path_flat (o->ctx);
+    Py_END_ALLOW_THREADS
+    return PycairoPath_FromPath (cp);
 }
 
 static PyObject *
@@ -287,7 +281,9 @@ pycairo_device_to_user_distance (PycairoContext *o, PyObject *args)
 static PyObject *
 pycairo_fill (PycairoContext *o)
 {
+    Py_BEGIN_ALLOW_THREADS
     cairo_fill (o->ctx);
+    Py_END_ALLOW_THREADS
     RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
     Py_RETURN_NONE;
 }
@@ -304,7 +300,9 @@ pycairo_fill_extents (PycairoContext *o)
 static PyObject *
 pycairo_fill_preserve (PycairoContext *o)
 {
+    Py_BEGIN_ALLOW_THREADS
     cairo_fill_preserve (o->ctx);
+    Py_END_ALLOW_THREADS
     RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
     Py_RETURN_NONE;
 }
@@ -460,7 +458,7 @@ static PyObject *
 pycairo_get_source (PycairoContext *o)
 {
     return PycairoPattern_FromPattern (
-	       cairo_pattern_reference (cairo_get_source (o->ctx)));
+	       cairo_pattern_reference (cairo_get_source (o->ctx)), NULL);
 }
 
 static PyObject *
@@ -639,7 +637,9 @@ pycairo_mask (PycairoContext *o, PyObject *args)
     if (!PyArg_ParseTuple(args, "O!:Context.mask", &PycairoPattern_Type, &p))
 	return NULL;
 
+    Py_BEGIN_ALLOW_THREADS
     cairo_mask (o->ctx, p->pattern);
+    Py_END_ALLOW_THREADS
     RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
     Py_RETURN_NONE;
 }
@@ -654,7 +654,9 @@ pycairo_mask_surface (PycairoContext *o, PyObject *args)
 			   &PycairoSurface_Type, &s, &surface_x, &surface_y))
 	return NULL;
 
+    Py_BEGIN_ALLOW_THREADS
     cairo_mask_surface (o->ctx, s->surface, surface_x, surface_y);
+    Py_END_ALLOW_THREADS
     RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
     Py_RETURN_NONE;
 }
@@ -691,7 +693,9 @@ pycairo_new_sub_path (PycairoContext *o)
 static PyObject *
 pycairo_paint (PycairoContext *o)
 {
+    Py_BEGIN_ALLOW_THREADS
     cairo_paint (o->ctx);
+    Py_END_ALLOW_THREADS
     RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
     Py_RETURN_NONE;
 }
@@ -704,7 +708,9 @@ pycairo_paint_with_alpha (PycairoContext *o, PyObject *args)
     if (!PyArg_ParseTuple (args, "d:Context.paint_with_alpha", &alpha))
 	return NULL;
 
+    Py_BEGIN_ALLOW_THREADS
     cairo_paint_with_alpha (o->ctx, alpha);
+    Py_END_ALLOW_THREADS
     RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
     Py_RETURN_NONE;
 }
@@ -721,7 +727,7 @@ pycairo_path_extents (PycairoContext *o)
 static PyObject *
 pycairo_pop_group (PycairoContext *o)
 {
-    return PycairoPattern_FromPattern (cairo_pop_group (o->ctx));
+    return PycairoPattern_FromPattern (cairo_pop_group (o->ctx), NULL);
 }
 
 static PyObject *
@@ -860,19 +866,33 @@ static PyObject *
 pycairo_select_font_face (PycairoContext *o, PyObject *args)
 {
     PyObject *obj;
-    const char *family;
-    cairo_font_slant_t slant = CAIRO_FONT_SLANT_NORMAL;
+    PyObject *pyUTF8 = NULL;
+    const char *utf8family = NULL;
+    cairo_font_slant_t slant   = CAIRO_FONT_SLANT_NORMAL;
     cairo_font_weight_t weight = CAIRO_FONT_WEIGHT_NORMAL;
 
     if (!PyArg_ParseTuple(args, "O!|ii:Context.select_font_face",
 			  &PyBaseString_Type, &obj, &slant, &weight))
 	return NULL;
 
-    family = __PyBaseString_AsUTF8 (obj);
-    if (family == NULL)
+    /* accept str and unicode family, auto convert to utf8 as required */
+    if (PyString_Check(obj)) {
+	/* A plain ASCII string is also a valid UTF-8 string */
+	utf8family = PyString_AS_STRING(obj);
+    } else if (PyUnicode_Check(obj)) {
+	pyUTF8 = PyUnicode_AsUTF8String(obj);
+	if (pyUTF8 != NULL) {
+	    utf8family = PyString_AS_STRING(pyUTF8);
+	}
+    } else {
+	PyErr_SetString(PyExc_TypeError,
+	    "Context.select_font_face: family must be str or unicode");
+    }
+    if (utf8family == NULL)
 	return NULL;
 
-    cairo_select_font_face (o->ctx, family, slant, weight);
+    cairo_select_font_face (o->ctx, utf8family, slant, weight);
+    Py_XDECREF(pyUTF8);
     RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
     Py_RETURN_NONE;
 }
@@ -940,20 +960,6 @@ pycairo_set_fill_rule (PycairoContext *o, PyObject *args)
     Py_RETURN_NONE;
 }
 
-static PyObject *
-pycairo_set_font_matrix (PycairoContext *o, PyObject *args)
-{
-    PycairoMatrix *matrix;
-
-    if (!PyArg_ParseTuple (args, "O!:Context.set_font_matrix",
-			   &PycairoMatrix_Type, &matrix))
-	return NULL;
-
-    cairo_set_font_matrix (o->ctx, &matrix->matrix);
-    RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
-    Py_RETURN_NONE;
-}
-
 static PyObject *
 pycairo_set_font_face (PycairoContext *o, PyObject *obj)
 {
@@ -971,6 +977,20 @@ pycairo_set_font_face (PycairoContext *o, PyObject *obj)
     Py_RETURN_NONE;
 }
 
+static PyObject *
+pycairo_set_font_matrix (PycairoContext *o, PyObject *args)
+{
+    PycairoMatrix *matrix;
+
+    if (!PyArg_ParseTuple (args, "O!:Context.set_font_matrix",
+			   &PycairoMatrix_Type, &matrix))
+	return NULL;
+
+    cairo_set_font_matrix (o->ctx, &matrix->matrix);
+    RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
+    Py_RETURN_NONE;
+}
+
 static PyObject *
 pycairo_set_font_options (PycairoContext *o, PyObject *args)
 {
@@ -1077,6 +1097,19 @@ pycairo_set_operator(PycairoContext *o, PyObject *args)
     Py_RETURN_NONE;
 }
 
+static PyObject *
+pycairo_set_scaled_font(PycairoContext *o, PyObject *args)
+{
+    PycairoScaledFont *f;
+    if (!PyArg_ParseTuple( args, "O!:Context.set_scaled_font",
+                           &PycairoScaledFont_Type, &f))
+        return NULL;
+
+    cairo_set_scaled_font(o->ctx, f->scaled_font);
+    RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
+    Py_RETURN_NONE;
+}
+
 static PyObject *
 pycairo_set_source (PycairoContext *o, PyObject *args)
 {
@@ -1160,7 +1193,9 @@ pycairo_show_glyphs (PycairoContext *o, PyObject *args)
     glyphs = _PyGlyphs_AsGlyphs (py_object, &num_glyphs);
     if (glyphs == NULL)
 	return NULL;
+    Py_BEGIN_ALLOW_THREADS
     cairo_show_glyphs (o->ctx, glyphs, num_glyphs);
+    Py_END_ALLOW_THREADS
     PyMem_Free (glyphs);
     RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
     Py_RETURN_NONE;
@@ -1169,7 +1204,9 @@ pycairo_show_glyphs (PycairoContext *o, PyObject *args)
 static PyObject *
 pycairo_show_page (PycairoContext *o)
 {
+    Py_BEGIN_ALLOW_THREADS
     cairo_show_page (o->ctx);
+    Py_END_ALLOW_THREADS
     RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
     Py_RETURN_NONE;
 }
@@ -1177,15 +1214,29 @@ pycairo_show_page (PycairoContext *o)
 static PyObject *
 pycairo_show_text (PycairoContext *o, PyObject *obj)
 {
-    const char *utf8 = __PyBaseString_AsUTF8 (obj);
-    if (utf8==NULL) {
-	PyErr_SetString(PyExc_TypeError,
-			"Context.show_text() argument must be a string or "
-			"unicode object");
-	return NULL;
-    }
+    PyObject *pyUTF8 = NULL;
+    const char *utf8 = NULL;
 
+    /* accept str and unicode text, auto convert to utf8 as required */
+    if (PyString_Check(obj)) {
+	/* A plain ASCII string is also a valid UTF-8 string */
+	utf8 = PyString_AS_STRING(obj);
+    } else if (PyUnicode_Check(obj)) {
+	pyUTF8 = PyUnicode_AsUTF8String(obj);
+	if (pyUTF8 != NULL) {
+	    utf8 = PyString_AS_STRING(pyUTF8);
+	}
+    } else {
+	PyErr_SetString(PyExc_TypeError,
+	    "Context.show_text: text must be str or unicode");
+    }
+    if (utf8 == NULL)
+	return NULL;
+
+    Py_BEGIN_ALLOW_THREADS
     cairo_show_text (o->ctx, utf8);
+    Py_END_ALLOW_THREADS
+    Py_XDECREF(pyUTF8);
     RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
     Py_RETURN_NONE;
 }
@@ -1193,7 +1244,9 @@ pycairo_show_text (PycairoContext *o, PyObject *obj)
 static PyObject *
 pycairo_stroke (PycairoContext *o)
 {
+    Py_BEGIN_ALLOW_THREADS
     cairo_stroke (o->ctx);
+    Py_END_ALLOW_THREADS
     RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
     Py_RETURN_NONE;
 }
@@ -1210,7 +1263,9 @@ pycairo_stroke_extents (PycairoContext *o)
 static PyObject *
 pycairo_stroke_preserve (PycairoContext *o)
 {
+    Py_BEGIN_ALLOW_THREADS
     cairo_stroke_preserve (o->ctx);
+    Py_END_ALLOW_THREADS
     RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
     Py_RETURN_NONE;
 }
@@ -1219,15 +1274,27 @@ static PyObject *
 pycairo_text_extents (PycairoContext *o, PyObject *obj)
 {
     cairo_text_extents_t extents;
-    const char *utf8 = __PyBaseString_AsUTF8 (obj);
-    if (utf8==NULL) {
+    PyObject *pyUTF8 = NULL;
+    const char *utf8 = NULL;
+
+    /* accept str and unicode text, auto convert to utf8 as required */
+    if (PyString_Check(obj)) {
+	/* A plain ASCII string is also a valid UTF-8 string */
+	utf8 = PyString_AS_STRING(obj);
+    } else if (PyUnicode_Check(obj)) {
+	pyUTF8 = PyUnicode_AsUTF8String(obj);
+	if (pyUTF8 != NULL) {
+	    utf8 = PyString_AS_STRING(pyUTF8);
+	}
+    } else {
 	PyErr_SetString(PyExc_TypeError,
-			"Context.text_extents() argument must be a string or "
-			"unicode object");
-	return NULL;
+	    "Context.text_extents: text must be str or unicode");
     }
+    if (utf8 == NULL)
+	return NULL;
 
     cairo_text_extents (o->ctx, utf8, &extents);
+    Py_XDECREF(pyUTF8);
     RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
     return Py_BuildValue("(dddddd)", extents.x_bearing, extents.y_bearing,
 			 extents.width, extents.height, extents.x_advance,
@@ -1237,28 +1304,27 @@ pycairo_text_extents (PycairoContext *o, PyObject *obj)
 static PyObject *
 pycairo_text_path (PycairoContext *o, PyObject *obj)
 {
-    const char *utf8 = __PyBaseString_AsUTF8 (obj);
-    if (utf8==NULL) {
+    PyObject *pyUTF8 = NULL;
+    const char *utf8 = NULL;
+
+    /* accept str and unicode text, auto convert to utf8 as required */
+    if (PyString_Check(obj)) {
+	/* A plain ASCII string is also a valid UTF-8 string */
+	utf8 = PyString_AS_STRING(obj);
+    } else if (PyUnicode_Check(obj)) {
+	pyUTF8 = PyUnicode_AsUTF8String(obj);
+	if (pyUTF8 != NULL) {
+	    utf8 = PyString_AS_STRING(pyUTF8);
+	}
+    } else {
 	PyErr_SetString(PyExc_TypeError,
-			"Context.text_path() argument must be a string or "
-			"unicode object");
-	return NULL;
+	    "Context.text_path: text must be str or unicode");
     }
+    if (utf8 == NULL)
+	return NULL;
 
     cairo_text_path (o->ctx, utf8);
-    RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
-    Py_RETURN_NONE;
-}
-
-static PyObject *
-pycairo_translate (PycairoContext *o, PyObject *args)
-{
-    double tx, ty;
-
-    if (!PyArg_ParseTuple (args, "dd:Context.translate", &tx, &ty))
-	return NULL;
-
-    cairo_translate (o->ctx, tx, ty);
+    Py_XDECREF(pyUTF8);
     RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
     Py_RETURN_NONE;
 }
@@ -1277,6 +1343,19 @@ pycairo_transform (PycairoContext *o, PyObject *args)
     Py_RETURN_NONE;
 }
 
+static PyObject *
+pycairo_translate (PycairoContext *o, PyObject *args)
+{
+    double tx, ty;
+
+    if (!PyArg_ParseTuple (args, "dd:Context.translate", &tx, &ty))
+	return NULL;
+
+    cairo_translate (o->ctx, tx, ty);
+    RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
+    Py_RETURN_NONE;
+}
+
 static PyObject *
 pycairo_user_to_device (PycairoContext *o, PyObject *args)
 {
@@ -1399,7 +1478,7 @@ static PyMethodDef pycairo_methods[] = {
     {"set_matrix",      (PyCFunction)pycairo_set_matrix,       METH_VARARGS},
     {"set_miter_limit", (PyCFunction)pycairo_set_miter_limit,  METH_VARARGS},
     {"set_operator",    (PyCFunction)pycairo_set_operator,     METH_VARARGS},
-    /* set_scaled_font - not implemented yet */
+    {"set_scaled_font", (PyCFunction)pycairo_set_scaled_font,  METH_VARARGS},
     {"set_source",      (PyCFunction)pycairo_set_source,       METH_VARARGS},
     {"set_source_rgb",  (PyCFunction)pycairo_set_source_rgb,   METH_VARARGS},
     {"set_source_rgba", (PyCFunction)pycairo_set_source_rgba,  METH_VARARGS},
